' bootstrap structural VAR impulse responses for Blanchard-Perotti model 

delete lvl_upp* lvl_low* lvl_med* acc_upp* acc_low* acc_med* rsp_lvl* rsp_acc*
include sub_bootirf.prg

'set variables
%VAR = "tax g_gov_cons_inv ngdp"  ' for BP   "tax gcn d_gdp"
!var = 3

%startdate =   "1978q1"   '  "1960q1"  "1975q1"   "1978q1"
%enddate =  "1997q4"   '   "1997q4"  "2006q4"  "2012q4"
smpl %startdate %enddate
'change path to program path
%path = @runpath
cd %path

'create dummy variable for 1975q2
'series dummy75 = @recode(@date=@dateval("1975Q2"), 1, 0)

'choose series depending on whether you need federal government, general government as in MR or on federal level tot_receipts - int_receipts 
series tax = bp_tax  '  mr_g_tax  mr_f_tax  my_tax bp_tax

series l_tt =  log(tax/(pop_fred*gdp_deflator))  ' for BP data log(tax/(pop*pgdp))
series l_gg =  log(g_gov_cons_inv/(pop_fred*gdp_deflator))  ' for BP data  log(g_gov_cons_inv/(pop*rgdp))  'log(gcn/(pop*pgdp))  
series l_yy = log(ngdp/(pop_fred*gdp_deflator))      '    log(ngdp/pop)     'for BP data     log(gdp/(pop*pgdp)) 
series ty = tax/ngdp   'adj_tax      ' for BP data tax/gdp
series gy = g_gov_cons_inv/ngdp  '  g_gov_cons_inv/ngdp   'for BP data gcn/gdp

'mean ratio for expressing impulses as shares of average gdp
scalar mr1 = @mean(ty)
scalar mr2 = @mean(gy)
scalar mr3 = 1

' estimate VAR blanchard-perotti
smpl %startdate %enddate
'var var_1.ls 1 4  l_gg l_tt l_yy  infl_q i
'var var_1.ls 1 4  l_tt l_gg l_yy @ dummy75(-4 to 0) @trend @trend^2 c
var var_1.ls 1 4  l_tt l_gg l_yy @ @trend @trend^2 c

' impose restrictions
var_1.cleartext(svar)

var_1.append(svar) @e1=2.08*@e3 + 0*@u2 + c(1)*@u1
var_1.append(svar) @e2= 0*@e3 + c(2)*@u1+c(3)*@u2
var_1.append(svar) @e3= c(4)*@e1+c(5)*@e2+c(6)*@u3

' estimate factorization
var_1.svar(rtype=text,conv=1e-5)

' set response horizon
!hrz = 20

' get residuals to bootstrap
var_1.makeresid(n=gres) res_l_tt res_l_gg res_l_yy

' normalizes initial impulse to 1% of average  gdp
		matrix mat1_a = var_1.@svaramat
		matrix mat1_b = var_1.@svarbmat
		matrix(!var,!var) mat1_b_diag
			for !i=1 to @columns(mat1_b)
				matplace(mat1_b_diag,@columnextract(mat1_b,!i)/mat1_b(!i,!i),1,!i)
			next

		matrix inv_mat1_a = @inverse(mat1_a)
		matrix mat1_e = inv_mat1_a*mat1_b
		matrix mat1_e_diag = inv_mat1_a*mat1_b_diag

' store level and accumulated responses
var_1.impulse(!hrz,m,imp=struct,matbys=rsp_lvl) l_tt l_gg l_yy @ l_tt l_gg l_yy
var_1.impulse(!hrz,m,imp=struct,a,matbys=rsp_acc) l_tt l_gg l_yy @ l_tt l_gg l_yy

' expresses impulse responses as shares of average gdp
	for !i=1 to @columns(rsp_lvl)/3
		!k = 0
		for !j=1 to @columns(rsp_lvl)/3
			vector tmp_col{!j} = @columnextract(rsp_lvl,!i+!k)
			matplace(rsp_lvl, tmp_col{!j}*mr{!i}, 1, !i+!k)
			!k = !k +3
		next
		delete tmp_col*
	next

	'percentage
	!k = 0
	for !j=1 to !var
		scalar prc{!j} = mat1_e_diag(!j,!j)/rsp_lvl(1,!j+!k*!var)
		!k = !k + 1
	next

	for !i=1 to @columns(rsp_lvl)/!var
		for !j=1 to !var
			vector tmp_col{!j} = @columnextract(rsp_lvl,(!i-1)*!var+!j)
			matplace(rsp_lvl, tmp_col{!j}*prc{!i}, 1, (!i-1)*!var+!j)
		next
		delete tmp_col*
	next

' make model to generate bootstrap data
var_1.makemodel(mod1)
' assign add factors for bootstrap residuals
mod1.addassign(i) @all

'set random number generator
rndseed(type=mt) 1234567
!reps = 1000
'declare storage matrices
for !i=1 to @columns(rsp_lvl)
	matrix(!hrz,!reps) brsp_lvl{!i}
	matrix(!hrz,!reps) brsp_acc{!i}
next

'temporary working vector
vector vtmp
'count number of cases that converged
scalar cnt = 0

'bootstrap loop
for !i=1 to !reps
	'bootstrap residuals
	'smpl @all if res_l_gg<>na
	'smpl @all if res_l_tt<>na
     smpl %startdate %enddate if res_l_tt<>na
     gres.resample l_tt_a l_gg_a l_yy_a
	'generate bootstrap data	
	mod1.solve
	'estimate VAR (no constant) with bootstrap data
	'smpl @all
	smpl %startdate %enddate

'var var_2.ls 1 4  l_tt_0 l_gg_0 l_yy_0  @ dummy75(-4 to 0) @trend @trend^2 c
var var_2.ls 1 4  l_tt_0 l_gg_0 l_yy_0 @ @trend @trend^2 c
' impose long-run restrictions
var_2.cleartext(svar)

var_2.append(svar) @e1=2.08*@e3 + 0*@u2 + c(1)*@u1
var_2.append(svar) @e2= 0*@e3 + c(2)*@u1+c(3)*@u2
var_2.append(svar) @e3= c(4)*@e1+c(5)*@e2+c(6)*@u3

	'estimate factorization
	var_2.svar(rtype=text,conv=1e-5,maxiter=100,nostop)
	'store results only if converged
	if (var_2.@svarcvgtype=0) then
		'increment counter
		cnt = cnt + 1
		'get level and accumulated bootstrap responses

           do var_2.impulse(!hrz,imp=struct,matbys=tmp_lvl)
		do var_2.impulse(!hrz,imp=struct,a,matbys=tmp_acc)
	
	' expresses impulse responses as shares of average gdp
	for !i_inc=1 to @columns(tmp_lvl)/3
		!k = 0
		for !j_inc=1 to @columns(tmp_lvl)/3
			vector tmp_col{!j_inc} = @columnextract(tmp_lvl,!i_inc+!k)
			matplace(tmp_lvl, tmp_col{!j_inc}*mr{!i_inc}, 1, !i_inc+!k)
			!k = !k +3
		next
		delete tmp_col*
	next

' normalizes initial impulse to 1% of average  gdp
		for !i_inc=1 to @columns(rsp_lvl)/!var
			for !j_inc=1 to !var
				vector tmp_col{!j_inc} = @columnextract(tmp_lvl,(!i_inc-1)*!var+!j_inc)
				matplace(tmp_lvl, tmp_col{!j_inc}*prc{!i_inc}, 1, (!i_inc-1)*!var+!j_inc)
			next
		delete tmp_col*
		next
	
		'tmp_lvl = tmp_lvl*prc1

		
    		'and store in appropriate position
		for !j=1 to @columns(rsp_lvl)
			vtmp = @columnextract(tmp_lvl,!j)
			colplace(brsp_lvl{!j}, vtmp, cnt)
			vtmp = @columnextract(tmp_acc,!j)
			colplace(brsp_acc{!j}, vtmp, cnt)
		next
		delete tmp_lvl tmp_acc
	endif
next

'reshape results matrix if there were non-converged cases
if (cnt <> !reps) then
	for !i=1 to @columns(rsp_lvl)
		brsp_lvl{!i} = @subextract(brsp_lvl{!i}, 1, 1, !hrz, cnt)
		brsp_acc{!i} = @subextract(brsp_acc{!i}, 1, 1, !hrz, cnt)
	next
	table warning
	%msg = @str(!reps-cnt) + " case(s) out of " + @str(!reps) + " replications failed to converge"
	warning(1,1) = %msg
	show warning
endif

'find bootstrap percentiles for each horizon
matrix(!hrz,@columns(rsp_lvl)) lvl_upp
matrix(!hrz,@columns(rsp_lvl)) lvl_med
matrix(!hrz,@columns(rsp_lvl)) lvl_low

matrix(!hrz,@columns(rsp_lvl)) lvl_upp1
matrix(!hrz,@columns(rsp_lvl)) lvl_med1
matrix(!hrz,@columns(rsp_lvl)) lvl_low1

matrix(!hrz,@columns(rsp_lvl)) acc_upp
matrix(!hrz,@columns(rsp_lvl)) acc_med
matrix(!hrz,@columns(rsp_lvl)) acc_low

rowvector rtmp
for !i=1 to !hrz
	for !j=1 to @columns(rsp_lvl)
		rtmp = @rowextract(brsp_lvl{!j},!i)
		lvl_upp(!i,!j) = @quantile(rtmp,0.833)
		lvl_med(!i,!j) = @quantile(rtmp,0.5)
		lvl_low(!i,!j) = @quantile(rtmp,0.167)
		rtmp = @rowextract(brsp_acc{!j},!i)
		acc_upp(!i,!j) = @quantile(rtmp,0.833)
		acc_med(!i,!j) = @quantile(rtmp,0.5)
		acc_low(!i,!j) = @quantile(rtmp,0.167)
	next
next

delete brsp_* res_*
'delete l_yy_0 l_tt_0 l_gg_0 infl_q_0 i_0 
'delete l_yy_a l_tt_a l_gg_a infl_q_a i_a

delete l_yy_0 l_tt_0 l_gg_0
delete l_yy_a l_tt_a l_gg_a
delete fig_lvl* tmp_graph*

'rescaling
'smpl 1960q1 1997q4
'scalar a1 = @mean(ty)
'lvl_med1 = lvl_med*a1
'lvl_upp1 = lvl_upp*a1
'lvl_low1 = lvl_low*a1

'plot impulse responses with bootstrap error bounds
%gname = "fig_lvl"
'call sub_bootirf(rsp_lvl, lvl_upp, lvl_low, %gname)
call sub_bootirf(lvl_med, lvl_upp, lvl_low, %gname)
' add zero line to all graphs

{%gname}.draw(line,left,rgb(180,180,180)) 0
' add title to graph
'{%gname}.addtext(0.1,-0.2) Bootstrap 66% Confidence Bounds (Levels)
' realign
'{%gname}.align(3,1,0.5)
{%gname}.align(2,1,1.5)
{%gname}.addtext(t,font("Times New Roman",22,b)) "Estimated Impact of tax and expenditure shocks in SVAR model" 
show {%gname}
delete fig_acc* tmp_graph*

%gname = "fig_acc"
'call sub_bootirf(rsp_acc, acc_upp, acc_low, %gname)
call sub_bootirf(acc_med, acc_upp, acc_low, %gname)
' add zero line to all graphs
{%gname}.draw(line,left,rgb(180,180,180)) 0
' add title to graph
'{%gname}.addtext(0.1,-0.2) Bootstrap 66% Confidence Bounds (Accumulated Responses)
' realign
'{%gname}.align(3,1,0.5)
{%gname}.align(2,1,1.5)
show {%gname}
