'***********************************************************************************
'*     DATA PREPARATION       *
'***********************************************************************************
'Loading data and transforming Excel file 
wfopen(page = INPUT) data.xlsx range="INPUT"
pageload(page = FRED_M) data.xlsx range="FRED_M"
pageload(page = BP_data) data.xlsx range="BP_data"

'set variables
%PAGE =   "INPUT"  
%FILE = "DATA"

wfselect {%FILE}\{%PAGE}
smpl @all 

'*************************************************************************************************
' CREATING  FINAL SERIES TO BE USED
'*************************************************************************************************
link tb3ms.linkto(c=a) FRED_M::tb3ms
link gs10.linkto(c=a) FRED_M::gs10
link cpi.linkto(c=l) FRED_M::cpi

series tb3ms1=tb3ms
series gs101=gs10
series cpi1=cpi
series d_cpi = log(cpi1/cpi1(-1))*100
series ne = netexp_fred/ngdp

series l_gdp = log(rgdp)*100
series d_gdp = d(log(rgdp)*100)
l_gdp.hpf(lambda = 100) gdp_hp
series gdp_c = l_gdp - gdp_hp

series l_debt = log(fygfdpun/(ngdp*(10^3)))*100
series d_debt = d(log(fygfdpun/(ngdp*(10^3)))*100)   ' since fygfdpun is in mln of $, and ngdp is in bln of $

series adj_exp = (tot_expend-int_paym)/ngdp
series adj_tax = (tot_receip-int_receip)/ngdp
series d_exp = d(log(adj_exp)*100)
series d_tax = d(log(adj_tax)*100)

series mr_g_tax = g_cur_tax_receipts + g_gov_social_ins
series mr_f_tax = f_cur_tax_receipts + f_gov_social_ins - fed_res_b   ' like in Mertens and Ravn 2012
series my_tax = tot_receip-int_receip
series bp_tax = g_current_receipts - g_curr_transf__persons - g_int_payments_persons + g_inc_receipts_on_assets  ' like in Blanchard and Perotti 2002

' SHOCKS
series tot_su = sut + sug
series tot_sa = satt + sagg
FOR !i = 1 TO 16
		series tot_sa{!i} = sat{!i} + sag{!i}
NEXT

'TOTAL PLANS (sum of un and an components, announced but not executed yet)
'series tot_plan = tot_su
'FOR !i = 1 TO !horz
'		series tot_plan = tot_plan + tot_sa{!i}
'NEXT

'CREATION OF THE EB&TB PLANS
series TB_sut = tot_su*TB
series GB_sug = tot_su*GB
series TB_satt = tot_sa*TB
series GB_sagg = tot_sa*GB
FOR !i=1 TO 16
	series TB_sat{!i} = tot_sa{!i}*TB
	series GB_sag{!i} = tot_sa{!i}*GB
NEXT

series tott = sut + satt
series totg = sug + sagg
series tot = tott + totg

'Create a deficit not offset by the long run from the romer and romer series
series poslongrnrratio = longrnrratio
smpl if poslongrnrratio>0
poslongrnrratio = 0
smpl @all
series t_def_long_rr = deficnrratio + poslongrnrratio
smpl if t_def_long_rr <0
series t_def_long_rr = 0

'Extend Romer and Romer series up to 2012 (since in the extended dataset there are only deficit driven episodes, it is possible to reconstruct deficit driven Romer and Romer series)

smpl 1975q1 2007q4
series def_ext = deficnrratio
series long_ext = longrnrratio
smpl 2008q1 2012q4
def_ext = 0
long_ext = 0

smpl 2010q2 2010q2     ' PPACA HR 3590 and RA
	def_ext = 0
	FOR !i=1 TO 16
		def_ext = def_ext + sat{!i}
	NEXT

smpl @all
'Create a deficit not offset by the long run from the extended romer and romer series
series poslong_ext = long_ext
smpl if poslong_ext >0
poslong_ext = 0
smpl @all
series t_d_lr_ext = def_ext + poslong_ext
smpl if t_d_lr_ext <0
series t_d_lr_ext = 0

smpl @all
series exog_ext = def_ext + long_ext

'Creating DISPLAY_NAME
deficnrratio.displayname R&R Deficit driven Tax Changes on sample "1978q1
exogenrratio.displayname R&R Exogenous Tax Changes on sample "1978q1
t_def_long_rr.displayname R&R Deficit driven Tax Changes not offset by Long run on sample "1978q1

def_ext.displayname R&R Deficit driven Tax Changes on sample "1978q1
exog_ext.displayname R&R Exogenous Tax Changes on sample "1978q1
t_d_lr_ext.displayname R&R Deficit driven Tax Changes not offset by Long run on sample "1978q1

tott.displayname IMF Deficit Driven Tax Changes on sample "1978q1
tot.displayname IMF Fiscal Consolidation (sum of tax hikes and spending cuts) on sample "1978q1

d_exp.displayname Expenditure 'variable growth
d_tax.displayname Tax 'variable growth
d_gdp.displayname Output growth
d_debt.displayname Debt growth

'series debt_n = fygfdpun/10^3
'series exp_n = tot_expend-int_paym
'series tax_n = tot_receip-int_receip
'smpl 1978q1 1978q1
'series debt_check=debt_n
'smpl 1978q2 @last
'series debt_check = debt_n(-1) + exp_n - tax_n + int_paym
'smpl @all
